@@ -1,6 +1,8 @@
 # flake8: noqa: F401
 from .misc import get_function
 from .misc import get_kwargs_applicable_to_function
+from .misc import locate_channels_axis
 from .misc import move_axis
 from .misc import onnx_model_node_loader
 from .misc import to_xarray
+from .onnx_runner import SimpleModelRunner
